c
c ---------------------------------------------------------------
c
      subroutine c2v(card,value,ilatlon)
ccccccccccccccccccccccccccccccccccccccccccccccccccc
c - Subroutine for converting a 40 character
c - string into a value.  The string must
c - consist of 1,2 or 3 numbers, meaning
c - Degrees, Degrees&Minutes or Deg, Min & Sec     
c - 
c - This has nothing to do with Positive 
c - Longitude conventions.  It is purely a 
c - character-to-value conversion.  Longitude
c - conventions are handled outside this
c - subroutine.
c - 
c - The allowable characters in "card" are
c - spaces, numbers 1 through 9, decimal 
c - points, and one leading negative sign.
c -
c - Returned values of "value" will be forced into:
c -   ilatlon = 1 =>     -90 < value < +90
c -   ilatlon = 2 =>       0 < value < +360
ccccccccccccccccccccccccccccccccccccccccccccccccccc
      character*80 SCCSID
      character*40 card

      real*8  deg
      real*8  min
      real*8  sec
      real*8  value
      real*8  xval

      integer*4  bkt
      integer*4  ekt
      integer*4  iflag
      integer*4  ilatlon
      integer*4  ival
      integer*4  b(50)
      integer*4  e(50)

      integer   ilen 
      integer   lnblnk
      integer   ii


      SCCSID="@(#)c2v.f   	1.1	2004/01/16 NGS"

      bkt = 0
      ekt = 0
ccccccccccccccccccccccccccccccccccccccccccccccccccc 
c - Search for the beginnings
c - and endings of numbers, assuming only
c - that we'll find the numbers '0' through '9' as
c - well as spaces, ' ', and decimals '.', and
c - possibly a leading negative sign '-'.
c - 
c - Comma delimeted data will not work
c - A space between the negative sign and the
c - first number is not allowable.
ccccccccccccccccccccccccccccccccccccccccccccccccccc 

      ilen = lnblnk(card)
ccccccccccccccccccccccccccccccccccccccccccccccccccc 
c - The position of the character which begins
c - the first 'number' is b(1), and the position
c - of the character which ends the first 'number'
c - is e(1).....etc for all numbers.
ccccccccccccccccccccccccccccccccccccccccccccccccccc 
      if(card(1:1).ne.' ')then
        bkt = bkt + 1
        b(bkt) = 1
      endif
 
      do 1 ii=2,ilen
        if(card(ii:ii).eq.' ')then
          if(card(ii-1:ii-1).ne.' ')then
            ekt = ekt + 1
            e(ekt) = ii-1
          endif
        else
          if(card(ii-1:ii-1).eq.' ')then
            bkt = bkt + 1
            b(bkt) = ii
          endif
        endif
    1 continue

c - Count the last space as an end, since we've
c - already defined its length as the last
c - non-space character
      ekt = ekt + 1
      e(ekt) = ii-1

ccccccccccccccccccccccccccccccccccccccccccccccccccc 
c - Obviously the number of beginnings and 
c - endings must be the same.
ccccccccccccccccccccccccccccccccccccccccccccccccccc 
      if(bkt .ne. ekt)stop 90909

ccccccccccccccccccccccccccccccccccccccccccccccccccc 
c - Integer/Decimal Degrees (1 number)
ccccccccccccccccccccccccccccccccccccccccccccccccccc 
      if(bkt .eq. 1)then
c - The following line doesn't work with older FORTRAN compilers
c       read(card,*)deg

        call val(card(b(1):e(1)),iflag,ival,xval)
        if(iflag.eq.0)deg = ival
        if(iflag.eq.1)deg = xval

        value = deg

ccccccccccccccccccccccccccccccccccccccccccccccccccc 
c - Integer Degrees & Integer/Decimal 
c - minutes (2 numbers)
c - Take care with pos/neg signs
ccccccccccccccccccccccccccccccccccccccccccccccccccc 
      elseif(bkt .eq. 2)then
c - The following line doesn't work with older FORTRAN compilers
c       read(card,*)deg,min

        call val(card(b(1):e(1)),iflag,ival,xval)
        if(iflag.eq.0)deg = ival
        if(iflag.eq.1)deg = xval

        call val(card(b(2):e(2)),iflag,ival,xval)
        if(iflag.eq.0)min = ival
        if(iflag.eq.1)min = xval
        if(deg.lt.0)min = -min 

        value = deg+(min/60.d0)

ccccccccccccccccccccccccccccccccccccccccccccccccccc 
c - Integer Degrees, Integer Minutes & 
c - Integer/Decimal Seconds (3 numbers)
c - Take care with pos/neg signs
ccccccccccccccccccccccccccccccccccccccccccccccccccc 
      elseif(bkt .eq. 3)then
c - The following line doesn't work with older FORTRAN compilers
c       read(ecard,*)deg,min,sec

        call val(card(b(1):e(1)),iflag,ival,xval)
        if(iflag.eq.0)deg = ival
        if(iflag.eq.1)deg = xval

        call val(card(b(2):e(2)),iflag,ival,xval)
        if(iflag.eq.0)min = ival
        if(iflag.eq.1)min = xval
        if(deg.lt.0)min = -min 

        call val(card(b(3):e(3)),iflag,ival,xval)
        if(iflag.eq.0)sec = ival
        if(iflag.eq.1)sec = xval
        if(deg.lt.0)sec = -sec 

        value = deg+(min/60.d0)+(sec/3600.d0)

ccccccccccccccccccccccccccccccccccccccccccccccccccc       
c - Otherwise there's an error and the value 
c - must be re-input
ccccccccccccccccccccccccccccccccccccccccccccccccccc 
      else
        value = -999.d0
      endif

ccccccccccccccccccccccccccccccccccccccccccccccccccc       
c - If, somehow, the latitude isn't in the
c - right range, turn lat/lon into -999s.
c - If somehow, the longitude isn't in the
c - right range, try a quick arithmetic trick,
c - and then if it's still bad, turn the 
c - lat/lon values into -999s
ccccccccccccccccccccccccccccccccccccccccccccccccccc 
c - Latitude
      if(ilatlon.eq.1)then
        if(value.lt.-90 .or. value.gt.+90)value = -999.d0
c - Longitude
      else
        if(value.lt.000)value = value + 360.d0
        if(value.gt.360)value = value - 360.d0
        if(value.lt.000 .and. value.gt.360)value = -999.d0
      endif      

      return
      end
c
