C /ngslib/source/Geoid/Flib/SCCS/s.spline6.f

      subroutine spline6 (latdd,londd,lin,kk, interp)
********************************************************************************
* Spline interpolation using a 6x6 window around the given point(latdd, londd)
* The size of the window can be changed to any even number. 
* Odd numbers are excluded since their use leads to non-symmetry 
* in the distribution of data arount the interpolation point.
*   in - latdd  : North lat of point in question  units = decdeg
*   in - londd  : East  lon of point in question  units = decdeg
*   in - lin    : vector of file unit numbers, pointing to grid files
*                 Data files must be direct access. 
*   in - kk     : unit number of geoid grid  data file
*   out- interp : interpolated geoid height, units as per data file
********************************************************************************
      implicit none
      character*80 SCCSID
      real*8       latdd
      real*8       londd
      real*8       row_val
      real*8       col_val
      integer*4    lin(50)
      integer*4    kk
      real*4       interp

      integer      nn
      parameter   (nn = 6)                       ! size of interpolation window
      integer      DEBUG
      parameter   (DEBUG = 0)                    ! diagnostics print if != 0

      real*8       latMin                        ! header record data
      real*8       lonMin
      real*8       latDelta
      real*8       lonDelta
      integer*4    latRowNum
      integer*4    lonColNum
      integer*4    irec
      real*8       latMax                        ! calcd from header data
      real*8       lonMax                        ! calcd from header data

      integer*4    irown, icoln
      real*8       row_counter, col_counter
      integer*4    row, col
      real*4       xx
      real*4       yy

      real*4       f1,  f2,  f3,  f4,  f5,  f6
      real*4       f7,  f8,  f9,  f10, f11, f12
      real*4       f13, f14, f15, f16, f17, f18
      real*4       f19, f20, f21, f22, f23, f24
      real*4       f25, f26, f27, f28, f29, f30
      real*4       f31, f32, f33, f34, f35, f36

      real*8       AA(nn)
      real*8       RR(nn)
      real*8       QQ(nn)
      real*8       HC(nn)

      real*8      glamn(50)            ! grid header data, in common block
      real*8      glamx(50)
      real*8      glomn(50)
      real*8      glomx(50)
      real*8      dla(50)
      real*8      dlo(50)
      integer*4   nla(50)
      integer*4   nlo(50)
      integer*4   ikind(50)
      common     /grid1/ glamn,glamx,glomn,glomx
      common     /grid2/ dla,dlo,nla,nlo,ikind

      logical   reverse_bytes(50)
      common    /endian/reverse_bytes

C Functions
      real*4      frev4
      real*8      spline

      SCCSID="@(#)spline6.f	1.2	2008/05/27 NGS"

C Define some necessary parameters
C These header data elements are read in the main driver file,
C are in a common block, and are already checked for endian condition
      latMin    = glamn(kk)
      lonMin    = glomn(kk)
      latDelta  = dla(kk)
      lonDelta  = dlo(kk)
      latRowNum = nla(kk)
      lonColNum = nlo(kk)
      latMax    = latMin + latDelta * (latRowNum - 1)
      lonMax    = lonMin + lonDelta * (lonColNum - 1)

      if (DEBUG .ge. 2) then
        print *, 'kk          = ',kk
        print *, 'latMin      = ',latMin
        print *, 'lonMin      = ',lonMin
        print *, 'latDelta    = ',latDelta
        print *, 'lonDelta    = ',lonDelta
        print *, 'latRowNum   = ',latRowNum
        print *, 'lonColNum   = ',lonColNum
        print *, 'latMax      = ',latMax
        print *, 'lonMax      = ',lonMax
      endif


c --------------------------------------------------------------------
c Find the row/col of the nearest grid node to the lat/lon point
c This grid node is southwest from the lat/lon point
c (Row_counter,Col_counter) = exact (lat,lon) grid coord location (real*8)
c (Row,Col)                 = closest node (to sw) grid coord location (int)
c --------------------------------------------------------------------
C - Tested and verified by billw
      col_counter = ((londd-lonMin) / lonDelta) + 1  ! +1 =fortran index corr'n
      row_counter = ((latdd-latMin) / latDelta) + 1  ! +1 =fortran index corr'n

      row   = int( row_counter )            ! the row just south from latdd
      col   = int( col_counter )            ! the col just west  from londd
      irown = row                           ! the reference node row
      icoln = col                           ! the reference node col

      row_val = (row-1)*latDelta + latMin   ! lat just south(up) from latdd
      col_val = (col-1)*lonDelta + lonMin   ! lon just west      from londd

      yy = (row_counter - row) + 3.d0       ! 3 := 6x6 spline window
      xx = (col_counter - col) + 3.d0

cc - original code from interg: 
cc      irown = nint( (latdd-glamn(kk)) / dla(kk) ) + 1
cc      icoln = nint( (londd-glomn(kk)) / dlo(kk) ) + 1
cc
cc      xx = ( londd - (glomn(kk)+(icoln-2)*dlo(kk)) ) / dlo(kk)
cc      yy = ( latdd - (glamn(kk)+(irown-2)*dla(kk)) ) / dla(kk)

      if (DEBUG .gt. 2) then
        print *, 'row_counter = ',row_counter
        print *, 'col_counter = ',col_counter       
        print *, 'row         = ',row
        print *, 'col         = ',col
        print *, 'xx          = ',xx
        print *, 'yy          = ',yy
        print *, 'latdd       = ',latdd
        print *, 'londd       = ',londd
        print *, 'row_val     = ',row_val,
     +           '  (Nlat) should be: row_val < latdd'
        print *, 'col_val     = ',col_val,
     +           '  (Elon) should be: col_val < londd'
      endif

c --------------------------------------------------------------------
c At this point, the (irown,icoln) coordinate pair represent 
c that grid node which is south and west from the lat/lon point, 
c about which we must get a 6x6 data matrix for spline interpolation
c Extract 6x6 array and use it for the spline interpolation
c --------------------------------------------------------------------
      irec = 11 + (irown-3)*nlo(kk) + icoln - 2
      read(lin(kk),rec=irec)f1
      irec = 11 + (irown-3)*nlo(kk) + icoln - 1
      read(lin(kk),rec=irec)f2
      irec = 11 + (irown-3)*nlo(kk) + icoln 
      read(lin(kk),rec=irec)f3
      irec = 11 + (irown-3)*nlo(kk) + icoln + 1
      read(lin(kk),rec=irec)f4
      irec = 11 + (irown-3)*nlo(kk) + icoln + 2
      read(lin(kk),rec=irec)f5
      irec = 11 + (irown-3)*nlo(kk) + icoln + 3
      read(lin(kk),rec=irec)f6

      irec = 11 + (irown-2)*nlo(kk) + icoln - 2
      read(lin(kk),rec=irec)f7
      irec = 11 + (irown-2)*nlo(kk) + icoln - 1
      read(lin(kk),rec=irec)f8
      irec = 11 + (irown-2)*nlo(kk) + icoln
      read(lin(kk),rec=irec)f9
      irec = 11 + (irown-2)*nlo(kk) + icoln + 1 
      read(lin(kk),rec=irec)f10
      irec = 11 + (irown-2)*nlo(kk) + icoln + 2
      read(lin(kk),rec=irec)f11
      irec = 11 + (irown-2)*nlo(kk) + icoln + 3
      read(lin(kk),rec=irec)f12

      irec = 11 + (irown-1)*nlo(kk) + icoln - 2
      read(lin(kk),rec=irec)f13
      irec = 11 + (irown-1)*nlo(kk) + icoln - 1
      read(lin(kk),rec=irec)f14
      irec = 11 + (irown-1)*nlo(kk) + icoln 
      read(lin(kk),rec=irec)f15
      irec = 11 + (irown-1)*nlo(kk) + icoln + 1
      read(lin(kk),rec=irec)f16
      irec = 11 + (irown-1)*nlo(kk) + icoln + 2
      read(lin(kk),rec=irec)f17
      irec = 11 + (irown-1)*nlo(kk) + icoln + 3
      read(lin(kk),rec=irec)f18

      irec = 11 + (irown  )*nlo(kk) + icoln - 2
      read(lin(kk),rec=irec)f19
      irec = 11 + (irown  )*nlo(kk) + icoln - 1
      read(lin(kk),rec=irec)f20
      irec = 11 + (irown  )*nlo(kk) + icoln 
      read(lin(kk),rec=irec)f21
      irec = 11 + (irown  )*nlo(kk) + icoln + 1
      read(lin(kk),rec=irec)f22
      irec = 11 + (irown  )*nlo(kk) + icoln + 2
      read(lin(kk),rec=irec)f23
      irec = 11 + (irown  )*nlo(kk) + icoln + 3
      read(lin(kk),rec=irec)f24

      irec = 11 + (irown+1)*nlo(kk) + icoln - 2
      read(lin(kk),rec=irec)f25
      irec = 11 + (irown+1)*nlo(kk) + icoln - 1
      read(lin(kk),rec=irec)f26
      irec = 11 + (irown+1)*nlo(kk) + icoln 
      read(lin(kk),rec=irec)f27
      irec = 11 + (irown+1)*nlo(kk) + icoln + 1
      read(lin(kk),rec=irec)f28
      irec = 11 + (irown+1)*nlo(kk) + icoln + 2
      read(lin(kk),rec=irec)f29
      irec = 11 + (irown+1)*nlo(kk) + icoln + 3
      read(lin(kk),rec=irec)f30

      irec = 11 + (irown+2)*nlo(kk) + icoln - 2
      read(lin(kk),rec=irec)f31
      irec = 11 + (irown+2)*nlo(kk) + icoln - 1
      read(lin(kk),rec=irec)f32
      irec = 11 + (irown+2)*nlo(kk) + icoln 
      read(lin(kk),rec=irec)f33
      irec = 11 + (irown+2)*nlo(kk) + icoln + 1
      read(lin(kk),rec=irec)f34
      irec = 11 + (irown+2)*nlo(kk) + icoln + 2
      read(lin(kk),rec=irec)f35
      irec = 11 + (irown+2)*nlo(kk) + icoln + 3
      read(lin(kk),rec=irec)f36


c convert big endian values to little endian
      if(reverse_bytes(kk)) then
        f1=frev4(f1)
        f2=frev4(f2)
        f3=frev4(f3)
        f4=frev4(f4)
        f5=frev4(f5)
        f6=frev4(f6)
        f7=frev4(f7)
        f8=frev4(f8)
        f9=frev4(f9)
        f10=frev4(f10)
        f11=frev4(f11)
        f12=frev4(f12)
        f13=frev4(f13)
        f14=frev4(f14)
        f15=frev4(f15)
        f16=frev4(f16)
        f17=frev4(f17)
        f18=frev4(f18)
        f19=frev4(f19)
        f20=frev4(f20)
        f21=frev4(f21)
        f22=frev4(f22)
        f23=frev4(f23)
        f24=frev4(f24)
        f25=frev4(f25)
        f26=frev4(f26)
        f27=frev4(f27)
        f28=frev4(f28)
        f29=frev4(f29)
        f30=frev4(f30)
        f31=frev4(f31)
        f32=frev4(f32)
        f33=frev4(f33)
        f34=frev4(f34)
        f35=frev4(f35)
        f36=frev4(f36)
      endif


      if (DEBUG .gt. 4) then
        print *, ' '
        print *,'Grid data file view (north is down, east to the right)'
        write(6,101) irown-2, f1,  f2,  f3,  f4,  f5,  f6, 
     +               irown-1, f7,  f8,  f9,  f10, f11, f12, 
     +               irown,   f13, f14, f15, f16, f17, f18, 
     +               irown+1, f19, f20, f21, f22, f23, f24, 
     +               irown+2, f25, f26, f27, f28, f29, f30, 
     +               irown+3, f31, f32, f33, f34, f35, f36,
     +               col-2, col-1, col,  col+1, col+2, col+3 

  101     format(/
     +           i5, 2x, f8.4, 2x, f8.4, 2x, f8.4, 2x,'|',1x,
     +                   f8.4, 2x, f8.4, 2x, f8.4,  /
     +           i5, 2x, f8.4, 2x, f8.4, 2x, f8.4, 2x,'|',1x,
     +                   f8.4, 2x, f8.4, 2x, f8.4,  /
     +           i5, 2x, f8.4, 2x, f8.4, 2x, f8.4, 2x,'|',1x,
     +                   f8.4, 2x, f8.4, 2x, f8.4,  /
     +  '   --------------------------------------------------------',/
     +           i5, 2x, f8.4, 2x, f8.4, 2x, f8.4, 2x,'|',1x,
     +                   f8.4, 2x, f8.4, 2x, f8.4,  /
     +           i5, 2x, f8.4, 2x, f8.4, 2x, f8.4, 2x,'|',1x,
     +                   f8.4, 2x, f8.4, 2x, f8.4,  /
     +           i5, 2x, f8.4, 2x, f8.4, 2x, f8.4, 2x,'|',1x,
     +                   f8.4, 2x, f8.4, 2x, f8.4,  /
     +           7x, 3(i5, 5x), 2x,3(i5,5x),/ )
      endif


      AA(1) = f1
      AA(2) = f2
      AA(3) = f3
      AA(4) = f4
      AA(5) = f5
      AA(6) = f6
      call initsp(AA,nn,RR,QQ)
      HC(1) = spline (xx,AA,nn,RR)
      if (DEBUG .gt. 6)  print *, 'HC(1)    = ',HC(1)

      AA(1) = f7
      AA(2) = f8
      AA(3) = f9
      AA(4) = f10
      AA(5) = f11
      AA(6) = f12
      call initsp(AA,nn,RR,QQ)
      HC(2) = spline (xx,AA,nn,RR)
      if (DEBUG .gt. 6)  print *, 'HC(2)    = ',HC(2)

      AA(1) = f13
      AA(2) = f14
      AA(3) = f15
      AA(4) = f16
      AA(5) = f17
      AA(6) = f18
      call initsp(AA,nn,RR,QQ)
      HC(3) = spline (xx,AA,nn,RR)
      if (DEBUG .gt. 6)  print *, 'HC(3)    = ',HC(3)

      AA(1) = f19
      AA(2) = f20
      AA(3) = f21
      AA(4) = f22
      AA(5) = f23
      AA(6) = f24
      call initsp(AA,nn,RR,QQ)
      HC(4) = spline (xx,AA,nn,RR)
      if (DEBUG .gt. 6)  print *, 'HC(4)    = ',HC(4)

      AA(1) = f25
      AA(2) = f26
      AA(3) = f27
      AA(4) = f28
      AA(5) = f29
      AA(6) = f30
      call initsp(AA,nn,RR,QQ)
      HC(5) = spline (xx,AA,nn,RR)
      if (DEBUG .gt. 6)  print *, 'HC(5)    = ',HC(5)

      AA(1) = f31
      AA(2) = f32
      AA(3) = f33
      AA(4) = f34
      AA(5) = f35
      AA(6) = f36
      call initsp(AA,nn,RR,QQ)
      HC(6) = spline (xx,AA,nn,RR)
      if (DEBUG .gt. 6)  print *, 'HC(6)    = ',HC(6)

c variable interp is the spline interpolated value, returned by reference
      call initsp (HC, nn, RR, QQ)
      interp = spline (yy, HC, nn, RR)

      return
      end

